using System;
using System.Linq;
using System.Collections.Generic;
using System.Globalization;
using System.IO;

/// @author Teemu Vesikukka
/// @version 28.03.2025
/// <summary>
/// Käyttäjä voi hoitaa pankkiasioita ohjelman kautta
/// </summary>
public class Pankkiautomaatti
{
    private static string _polkuSaldo = @"C:\Users\Vespa\Desktop\gradia\ohjelmointi1_jyu_2025\harjoitustyo\Harjoitustyo\Pankkiautomaatti\saldo.txt";
    private static string _polkuTiliote = @"C:\Users\Vespa\Desktop\gradia\ohjelmointi1_jyu_2025\harjoitustyo\Harjoitustyo\Pankkiautomaatti\tiliote.txt";
    
    
    /// <summary>
    /// Pääohjelma joka kysyy pin-koodin kayttajalta ja pin-koodin ollessa oikein, paastaa
    /// asiakkaan omalle tilille
    /// </summary>
    public static void Main()
    {
        Aloitus();
        
        if (PinKoodi())
        {
            double alkusaldo = AlkuSaldo();
            
            List <string> tiliote = File.ReadAllLines("Tiliote.txt").ToList();
            Valikko(alkusaldo, tiliote);
        }
        Lopetus();
    }
    
    
    /// <summary>
    /// Ohjelman aloitus
    /// </summary>
    private static void Aloitus()
    {
        Console.WriteLine("Tervetuloa!");
        Console.WriteLine("Syota kortti ole hyva");
    }
    
    
    /// <summary>
    /// Aliohjelma lukee tekstitiedostosta tilin saldon
    /// </summary>
    /// <returns>saldo</returns>
    private static double AlkuSaldo()
    {
        string[] luetutRivit = File.ReadAllLines(_polkuSaldo);
        double saldo = Convert.ToDouble(luetutRivit[0]);
        return saldo;
    }
    
    
    /// <summary>
    /// Aliohjelma vie tekstitiedostoon tilin saldon
    /// </summary>
    /// <param name="loppuSaldo"></param>
    private static void LoppuSaldo(double loppuSaldo)
    {
        string [] saldo = new string[1];
        saldo[0] = loppuSaldo.ToString(CultureInfo.CurrentCulture);
        
        File.WriteAllLines(_polkuSaldo, saldo);
     
    }
    
    
    /// <summary>
    /// Aliohjelma vie tekstitiedostoon tilin käyttäjän nostot ja talletukset
    /// </summary>
    /// <param name="tiliote">tiliote</param>
    private static void LoppuTiliote(List<string> tiliote)
    {
        File.WriteAllLines(_polkuTiliote, tiliote);
    }
    
    
    /// <summary>
    /// Aliohjelma joka pyytaa asiakkaalta pin-koodin ja koodin
    /// ollessa oikein paastaa asiakkaan kayttamaan tilia
    /// </summary>
    /// <returns>palauttaa totuusarvon onko pinkoodi oikein</returns>
    /// <example>
    /// <pre name="test">
    /// PinKoodi(),"1234"===true;
    /// </pre>
    /// </example>
    private static bool PinKoodi()
    {
        int yritykset = 1;
        int pinkoodi = 1234;
        bool onkoOikien = true;
        Console.WriteLine("Anna pin koodi >" );
        int koodi = Convert.ToInt32(Console.ReadLine());
        yritykset++;
        if (koodi.Equals(pinkoodi))
        {
            
        }
        else
        {
            while (yritykset <= 3)
            {
                Console.WriteLine("Pinkoodi väärin");
                Console.WriteLine("Anna pin koodi >" );
                int koodi2 = Convert.ToInt32(Console.ReadLine());
                yritykset++;
                if (koodi2.Equals(pinkoodi))
                {
                    onkoOikien = true;
                    break;
                }
                else
                {
                    Console.WriteLine("Annoit kolme kertaa pin koodin vaarin");
                    Console.WriteLine("Kortti on otettu talteen");
                    onkoOikien = false; 
                }
            }
        }
        return onkoOikien;
    }
    
    
    /// <summary>
    /// Päävalikko josta hoidetaan tiliasiat
    /// </summary>
    /// <param name="alkusaldo">alkusaldo</param>
    /// <param name="tiliote">tiliote</param>
    private static void Valikko(double alkusaldo, List<string> tiliote)
    {
        double saldo = alkusaldo;
        while (true)
        {
            Console.WriteLine("Jos haluat saldon kirjoita saldo: ");
            Console.WriteLine("Jos haluat tiliotteen kirjoita ote");
            Console.WriteLine("Jos haluat nostaa kirjoita otto");
            Console.WriteLine("Jos haluat pysayttaa kirjoita stop");
            Console.WriteLine("Jos haluat tallettaa tilille rahaa kirjoita talletus");
            string tapahtuma = Console.ReadLine();
            if (tapahtuma == "saldo")
            {
                TulostaSaldo(saldo);
            }
            else if (tapahtuma == "ote")
            {
                TulostaTiliote(tiliote, saldo);
            }
            else if (tapahtuma == "otto")
            {
                saldo = Nosto(saldo, tiliote);
            }
            else if (tapahtuma == "talletus")
            {
                saldo = Talletus(saldo, tiliote);
            }
            else if(tapahtuma == "stop")
            {
                LoppuSaldo(saldo);
                LoppuTiliote(tiliote);
                break;
            }
        }
    }
    
    
    /// <summary>
    /// Funktio joka ottaa tililta rahaa asiakkaan nostaessa tililta rahaa
    /// Funktio palauttaa saldon
    /// Tilin saldon ollessa liian pieni tililta ei pysty nostamaan rahaa
    /// </summary>
    /// <param name="saldo">parametrina viedaan saldo</param>
    /// <param name="tiliote">parametrina viedaan tiliote</param>
    /// <returns>palauttaa tilin saldon</returns>
    private static double Nosto(double saldo, List<string> tiliote)
    {
        Console.WriteLine("Kuinka paljon haluat nostaa: ");
        double nosto = Convert.ToDouble(Console.ReadLine());
        
        if (saldo > 0.00 && nosto < saldo)
        {
            saldo = saldo - nosto;
            string rivi = DateTime.Now.ToString("dd/MM/yyyy")+ "\totto" + "\t" + " - "+ nosto.ToString(CultureInfo.CurrentCulture);
            tiliote.Add(rivi);
        }
        else
        {
            Console.WriteLine("Tilillä ei ole katetta");
        }

        return saldo;
    }
    
    
    /// <summary>
    /// funktio joka lisaa tilille asiakkaan lisaaman summan ja palauttaa saldon
    /// </summary>
    /// <param name="saldo">parametrina viedaan alkuperäinen saldo</param>
    /// <param name="tiliote">parametrina viedaan alkuperäinen tiliote</param>
    /// <returns>palauttaa lisatyn saldon</returns>
    private static double Talletus(double saldo, List<string> tiliote)
    {
        Console.Write("Kuinka paljon haluat tallettaa tilille: ");
        double talletus = Convert.ToDouble(Console.ReadLine());
        //double talletus = double.Parse(Console.ReadLine());

        saldo = saldo + talletus;
        string rivi = DateTime.Now.ToString("dd/MM/yyyy")+ "\ttalletus" + "\t" + " + " +talletus.ToString(CultureInfo.CurrentCulture);
        tiliote.Add(rivi);
        
        return saldo;
    }
  
    
    /// <summary>
    /// Aliohjelma joka tulostaa tlin saldon
    /// </summary>
    /// <param name="saldo">parametrina viety tilin saldo</param>
    private static void TulostaSaldo(double saldo)
    {
        Console.WriteLine("Tililla on rahaa " + " + " + saldo + " euroa");
        Console.WriteLine("Paavalikkoon paase painamalla enter");
        Console.ReadLine();
    }
    
    
    /// <summary>
    /// Aliohjelma joka tulostaa tilitapahtumat
    /// </summary>
    private static void TulostaTiliote(List<string> tiliote, double saldo)
    {
        int viimeinen = tiliote.Count - 1;
        string temp;
        for (int i = 1,j = viimeinen; i < j; i++,j--)
        {
            temp = tiliote[i];
            tiliote[i] = tiliote[j];
            tiliote[j] = temp;
        }

        int lkm = 0;
        while (lkm < tiliote.Count)
        {
            Console.WriteLine(tiliote[lkm]);
            lkm++;
        }  
        
        Console.WriteLine("Tilin saldo: {0,10:f2}\t", saldo);
        Console.WriteLine("alkuvalikkoon paaset painamalla enter");
        Console.ReadLine();
    }
    
    
    /// <summary>
    /// Aliohjelma joka tulostaa naytolle ohjelmastapoistuttaessa
    /// </summary>
    private static void Lopetus()
    {
        Console.WriteLine("Kiitos kaynnista");
        Console.WriteLine("Tervetuloa uudelleen");
    }
}